"""
    The ICOS CarbonPortal package provides access to data hosted at the
    Carbon Portal. ( https://data.icos-cp.eu/ )    
    
    This library will load binary representations of digital data objects
    directly into memory. If you need store the data locally, we suggest
    you go to the website and "dowload" the data. The approach of this library
    is to free you from dowloadin and maintaining a local copy of data files.
    
    The ICOS Carbon Portal provides persistent digital object identifiers. Hence
    you only need to store a list of pid's and your script to reproduce
    always the same result, regardless on which computer you run it.
    (as long as you have an internet connection)    
"""

__credits__     = "ICOS Carbon Portal"
__license__     = ['Software: GPL-3.0','ICSO Data accessed with this library CC BY 4.0']
__version__     = "0.1.9"
__maintainer__  = "ICOS Carbon Portal, elaborated products team"
__email__       = ['info@icos-cp.eu', 'claudio.donofrio@nateko.lu.se']
__status__      = "stable"
__date__        = "2021-10-11"

